import { compareMethods, } from '../_shared';
class BinaryHeap {
    constructor() {
        this.values = [];
        this.compare = compareMethods.compare.bind(this);
        this.inf = compareMethods.inf.bind(this);
        this.sup = compareMethods.sup.bind(this);
        this.equal = compareMethods.equal.bind(this);
        this.infOrEqual = compareMethods.infOrEqual.bind(this);
        this.supOrEqual = compareMethods.supOrEqual.bind(this);
    }
    setCompareFunction(compareFunction) {
        this.compare = compareFunction;
        const values = this.toArray();
        this.clear();
        this.insert(...values);
        return this;
    }
    get length() {
        return this.values.length;
    }
    toArray() {
        return this.values;
    }
    insert(...values) {
        values.forEach((value) => {
            this.values.push(value);
            this.bubbleUp();
        });
        return values[values.length - 1];
    }
    shift() {
        const first = this.values[0];
        if (first === undefined)
            return undefined;
        const last = this.values.pop();
        if (this.length === 0)
            return first;
        this.values[0] = last;
        this.bubbleDown();
        return first;
    }
    clear() {
        this.values = [];
    }
    bubbleUp() {
        let currentIndex = this.values.length - 1;
        while (currentIndex > 0) {
            const parentIndex = Math.floor((currentIndex - 1) / 2);
            if (this.infOrEqual(this.values[currentIndex], this.values[parentIndex]))
                break;
            this.swap(currentIndex, parentIndex);
            currentIndex = parentIndex;
        }
    }
    bubbleDown() {
        const maxValueIndex = (...indexes) => {
            let iMax = indexes[0];
            indexes.forEach((i) => {
                if (this.supOrEqual(this.values[i], this.values[iMax]))
                    iMax = i;
            });
            return iMax;
        };
        let i = 0;
        while (i < this.values.length) {
            const iLchild = 2 * i + 1;
            const iRchild = iLchild + 1;
            const iMax = (() => {
                if (!this.hasIndex(iLchild))
                    return i;
                if (!this.hasIndex(iRchild))
                    return maxValueIndex(i, iLchild);
                return maxValueIndex(i, iLchild, iRchild);
            })();
            const next = i === iMax ? 0 : iMax;
            if (!next)
                break;
            this.swap(i, next);
            i = next;
        }
    }
    swap(i, j) {
        [this.values[i], this.values[j]] = [this.values[j], this.values[i]];
    }
    hasIndex(i) {
        return i < this.values.length;
    }
}
export default BinaryHeap;
//# sourceMappingURL=binary-heap.js.map