import type { Comparer, CompareFunction, FilterFunction, MapFunction, ReduceFunction } from '../_shared';
import { TraverseMethod } from '../_shared';
declare class BinarySearchTreeNode<T> {
    value: T;
    left?: BinarySearchTreeNode<T>;
    right?: BinarySearchTreeNode<T>;
    constructor(value: T);
    hasChild(direction: -1 | 1): boolean;
    getChild(direction: -1 | 1): BinarySearchTreeNode<T> | undefined;
    setChild(direction: -1 | 1, newNode: BinarySearchTreeNode<T>): void;
}
declare class BinarySearchTree<T> implements Comparer<T> {
    private _root?;
    compare: CompareFunction<T>;
    inf: <T_1>(this: any, a: T_1, b: T_1) => boolean;
    sup: <T_1>(this: any, a: T_1, b: T_1) => boolean;
    equal: <T_1>(this: any, a: T_1, b: T_1) => boolean;
    infOrEqual: <T_1>(this: any, a: T_1, b: T_1) => boolean;
    supOrEqual: <T_1>(this: any, a: T_1, b: T_1) => boolean;
    private traverseMethods;
    defaultTraverseMethod: TraverseMethod;
    constructor(compareFunction?: CompareFunction<T>);
    root(): BinarySearchTreeNode<T> | undefined;
    setCompareFunction(compareFunction: CompareFunction<T>): BinarySearchTree<T>;
    setDefaultTraverseMethod(traverseMethod: TraverseMethod): void;
    insert(value: T): boolean;
    clear(): void;
    toArray(traverseMethod?: TraverseMethod): T[];
    map<U>(mapFunction: MapFunction<T, U>, traverseMethod?: TraverseMethod, newCompareFunction?: CompareFunction<U>): BinarySearchTree<U>;
    filter(filterFunction: FilterFunction<T>, traverseMethod?: TraverseMethod): BinarySearchTree<T>;
    reduce<U>(reduceFunction: ReduceFunction<T, U>, initialValue: U, traverseMethod?: TraverseMethod): U;
    private traverseBFS;
    private traverseDFS;
    private traversePreOrder;
    private traverseInOrder;
    private traversePostOrder;
}
export default BinarySearchTree;
export { TraverseMethod };
